use anyhow::Result;
use tokio::io::AsyncWrite;

use crate::handler::{IrcCommand, IrcConnection};
use lavina_core::player::GetInfoResult;
use lavina_core::player::PlayerId;
use lavina_core::prelude::Str;
use proto_irc::client::Whois;
use proto_irc::commands::whois::error::{ErrNoNicknameGiven431, ErrNoSuchNick401};
use proto_irc::commands::whois::response::RplEndOfWhois318;
use proto_irc::response::IrcResponseMessage;
use proto_irc::response::WriteResponse;

impl IrcCommand for Whois {
    async fn handle_with(&self, conn: &mut IrcConnection<'_, impl AsyncWrite + Unpin>) -> Result<()> {
        match self {
            Whois::Nick(nick) => handle_nick_target(nick.clone(), conn).await?,
            Whois::TargetNick(_, nick) => handle_nick_target(nick.clone(), conn).await?,
            Whois::EmptyArgs => {
                IrcResponseMessage::empty_tags(
                    Some(conn.server_name.clone()),
                    ErrNoNicknameGiven431::new(conn.server_name.clone()),
                )
                .write_response(conn.writer)
                .await?
            }
        }
        Ok(())
    }
}

async fn handle_nick_target(nick: Str, conn: &mut IrcConnection<'_, impl AsyncWrite + Unpin>) -> Result<()> {
    match conn.player_connection.check_user_existence(PlayerId::from(nick.clone())?).await? {
        GetInfoResult::UserExists => {}
        GetInfoResult::UserDoesntExist => {
            IrcResponseMessage::empty_tags(
                Some(conn.server_name.clone()),
                ErrNoSuchNick401::new(conn.client.clone(), nick.clone()),
            )
            .write_response(conn.writer)
            .await?
        }
    }
    IrcResponseMessage::empty_tags(
        Some(conn.server_name.clone()),
        RplEndOfWhois318::new(conn.client.clone(), nick.clone()),
    )
    .write_response(conn.writer)
    .await?;
    Ok(())
}
